'' System Version 2.002
'' OS_initialization Changes:
''
''   2006/09/24 - Moved from the main program to a separate object.  Added a parameter
''                for the initial display message.
''   2006/11/02 - Modified for use with the Hydra system
''   2006/11/03 - Now makes sure boot EEPROM is called "**BOOT**" internally
''   2006/11/04 - Call ldrInit.computeTimes before calling ldr.start
''   2006/11/06 - Initialize relocated on-board boot EEPROM area if not already done
''   2006/11/08 - Changed fatal error message editing
''   2006/11/10 - Now does no initialization if already done once
''   2006/11/13 - Keyboard and display pin groups moved to OS_loader
''   2006/11/27 - Added initialization for Vinculum USB Host
''   2006/12/08 - Cleaned up display initialization code

'' This is the initialization routine for the Propeller Operating System.  It gets its
'' hardware specific information from OS_loader, so this code is hardware independent.

OBJ
  ldr     : "OS_loader"                        ' SPIN program loader and support routines
  ldrInit : "OS_loaderInit"                    '  Initialization and assembly portion
  dsp     : "OS_dspText"                       ' Display text driver modified for OS use
  dspInit : "OS_dspTextInit"                   '  Initialization and assembly portion
  key     : "OS_keyboard"                      ' PS/2 keyboard driver modified for OS use
  keyInit : "OS_keyboardInit"                  '  Initialization and assembly portion
  files   : "OS_fileSystem"                    ' File system support
  mass    : "OS_massStorage"                   ' DOSonChip mass storage support

PUB start | t, i, p
'' Operating System initialization.  Only check for keyboard and presence of boot EEPROM
'' the first time.  A marker is placed in high memory (at ldr#initMarker) to indicate
'' that initialization has already occurred.  This is erased during the boot process.
  if long[ldr#initMarker] <> ldr#uniqueMark                ' Check for a valid initialization
    repeat i from 0 to 7
      if i <> cogid                                        ' Stop all cogs except this one
        cogstop(i)
    long[ldr#memPtr]    := ldr#endFree                     ' Start at end of HUB memory
    p := ldr.allocatePRI(ldr#systemSize)                   ' Allocate system work area
    long[ldr#systemPtr] := p
    long[p+ldr#sys_cog] := |<cogid                         ' Mark current cog in use
    byte[p+ldr#sys_cogMain] := cogid + 1
    long[p+ldr#sys_random] := ldr#initMarker               ' Initialize random seed
    wordfill(p+ldr#sys_eeproms,0,ldr#maxPinPairs)          ' Clear EEPROMs bitmap table
    long[ldr#videoPtr]  := ldr.allocatePRI(ldr#videoSize)  ' Allocate work area for video
    long[ldr#screenPtr] := ldr.allocatePRI(ldr#screenSize) ' Allocate screen buffer
    long[ldr#colorPtr]  := ldr.allocatePRI(ldr#colorSize)  ' Allocate screen color buffer
    long[ldr#fontPtr]   := ldr.allocatePRI(ldr#fontSize)   ' Allocate font table
    long[ldr#cursorPtr] := ldr.allocatePRI(ldr#cursorSize) ' Allocate cursor control block
    if not dspInit.start(ldr#dspPinGroup,ldr#device,ldr#cols,ldr#rows,ldr#tiles)
      abort false                                          ' Now start up the display driver
    showBanner                                             ' Display a "sign on" string
    long[ldr#keybdPtr]  := ldr.allocatePRI(ldr#keybdSize)  ' Allocate work area for keyboard
    if not keyInit.start(ldr#keyPinGroup)                  ' Now start up the keyboard driver
      abort false
    ldrInit.computeTimes                                   ' Initialize I2C timing values
    if not ldr.start(ldrInit.getEntryPoint)                ' Now start up the I2C driver
      abort false                                         
    long[ldr#userPtr] := ldr#noSuchAddr
    long[ldr#initMarker] := ldr#uniqueMark                 ' Basic initialization complete
    waitcnt(clkfreq * 2 + cnt)                             ' Wait for keyboard to initialize
    dsp.out(dsp#Cr)
    if not key.present
      abort initError(string("--> keyboard not present",dsp#Cr),0)      
    if (t := \files.updateBitmapEEPROMs(ldr#i2cBootSCL)) < 0
      return initError(string("--> can't access boot EEPROM!"),t)
    if (t & $4000) == 0                                    ' Check boot EEPROM present
      return initError(string("--> no boot EEPROM present"),0)
    if (t := \files.writeFilename(0,ldr#bootAddr,0)) < 0   ' Make sure boot EEPROM is **BOOT
      return initError(string("--> can't write boot EEPROM!"),t)
    if ldr.checkPresence(ldr#altBootAddr)                  ' See if cartridge in place
      if (t := \files.checkSums(string("**BOOT      "),ldr#altBootAddr)) < 0
        if t == true
          t := 0
        else
          return initError(string("--> error in alt boot EEPROM!"),t)
      if (t := \files.writeFilename(0,ldr#altBootAddr,t)) < 0
        return initError(string("--> can't write alt boot EEPROM!"),t)
 '   if (t := \mass.start) <> true
 '     case t
 '       mass#Timeout:
 '         return initError(string("--> mass storage Timeout"),0)
 '       mass#Invalid:
 '         return initError(string("--> mass storage Invalid"),0)
 's     return initError(string("--> mass storage CF"),0)
  return true

PRI initError(m,t)                                         '' Display error message
  dsp.str(m)                                               '   then disable OS
  if t <> 0
    dsp.str(string(dsp#Cr,"    "))
    dsp.str(-t)
  dsp.out(dsp#Cr)
  long[ldr#initMarker] := 0
  return false

PUB showBanner                                         '' Display an identification banner
  if ldr#Parallax
    dsp.str(string("Propeller"))
  if ldr#Hydra
    dsp.str(string("Hydra"))
  dsp.out(" ")
  if ldr#TV
    dsp.str(string("TV"))
  if ldr#VGA
    dsp.str(string("VGA"))
  dsp.str(string(" OS Version "))
  dsp.dec(ldr#OSversion)
  dsp.out(".")
  if ldr#OSrelease < 100
    dsp.out("0")
  if ldr#OSrelease < 10
    dsp.out("0")
  dsp.dec(ldr#OSrelease)
  if ldr#OStestLevel > 0
    dsp.out("a"+ldr#OStestLevel-1)
  dsp.str(string(dsp#Cr,"Type help for a command list"))