'' System Version 2.002
'' OS_massStorage Changes:
''
''   2006/11/27 - Initial version with change dates
''   2006/11/30 - Clean up command/response stuff

CON
  RESET     = 0                   ' Vinculum /Reset
  SPI_CLK   = 1                   ' Vinculum Clock
  SPI_DATA  = 2                   ' Vinculum Data
  SPI_CS    = 3                   ' Vinculum Select
  EOL       = 13                  ' Vinculum End of Line character
  prompt    = ">"                 ' Vinculum command prompt
  maxTime   = 100                 ' Default timeout is 100ms
  TimeOut   = $1111               ' Timeout error abort
  Invalid   = $2222               ' Invalid or unexpected response abort

OBJ
  ldr   : "OS_loader"             ' System definitions & I2C driver
  dsp   : "OS_dspText"            ' Display text driver modified for OS use

VAR
  long maxTimeOut
  
PUB start
  outa[RESET]~                    ' Reset device
  dira[RESET]~~
  outa[SPI_CS]~                   ' Select normally low
  dira[SPI_CS]~~
  outa[SPI_CLK]~                  ' Clock normally low
  dira[SPI_CLK]~~
  dira[SPI_DATA]~                 ' Data initially input
  outa[RESET]~~
  maxTimeOut := 0
  waitcnt(clkfreq / 4 + cnt)      ' Wait for 250ms, then check for prompts
  repeat until waitResponse(-maxTime*20,string("On-Line:",EOL))
  if not waitResponse(-maxTime*50,string(prompt,EOL))
    abort Invalid
  str(string("SCS",EOL))          ' Switch to shortened command set
  return waitPrompt(maxTime)

PRI transfer(read,status,data) | i, mask
'' Transfer data to/from Vinculum USB Host in SPI mode
  data := $800 | $400 & read | $200 & status | data << 1
  mask := $800
  outa[SPI_CLK]~~                 ' At least one clock prior to selecting
  outa[SPI_CLK]~
  outa[SPI_CS]~~                  ' Select chip
  repeat i from 1 to 12           ' Start, direction, address, 8 data, status
    dira[SPI_DATA] := i < 4 or i < 12 and not read
    outa[SPI_DATA] := (data & mask) > 0
    result |= mask & (ina[SPI_DATA] > 0)
    outa[SPI_CLK]~~               ' Data valid on positive edge of clock
    dira[SPI_DATA]~               ' Let data line float after hold time
    outa[SPI_CLK]~
    mask >>= 1
  outa[SPI_CS]~                   ' Deselect chip
  outa[SPI_CLK]~~                 ' At least one clock after deselecting
  outa[SPI_CLK]~

PUB out(c)                                          '' Output a single character
  sendChar(maxTime,c)

PUB str(stringptr)                                  '' Output a zero-terminated string
  repeat strsize(stringptr)
    out(byte[stringptr++])

PUB dec(value) | i                                  '' Output a decimal number
  if value < 0
    -value
    out("-")
  i := 1_000_000_000
  repeat 10
    if value => i
      out(value / i + "0")
      value //= i
      result~~
    elseif result or i == 1
      out("0")
    i /= 10

PUB hex(value, digits)                              '' Output a hexadecimal number
  value <<= (8 - digits) << 2
  repeat digits
    out(lookupz((value <-= 4) & $F : "0".."9", "A".."F"))

PUB bin(value, digits)                              '' Output a binary number
'' Output a binary number
  value <<= 32 - digits
  repeat digits
    out((value <-= 1) & 1 + "0")

PUB doCommand(s)                                    '' For testing, output any command
  str(||s)
  out(EOL)
  if s < 0                                          ' Then either echo the response or
    waitPrompt(-maxTime)
  else                                              '  wait for a standard prompt
    return waitPrompt(maxTime)

PUB deleteDir(s)                                    '' Delete specified subdirectory
  out($05)
  out(" ")
  str(s)
  out(EOL)
  return waitPrompt(maxTime)

PUB makeDir(s)                                      '' Make specified subdirectory
  out($07)
  out(" ")
  str(s)
  out(EOL)
  return waitPrompt(maxTime)

PUB changeDir(s)                                    '' Change to specified subdirectory
  out($02)
  out(" ")
  str(s)
  out(EOL)
  return waitPrompt(maxTime)

PUB openRead(s)                                     '' Open specified file for reading
  out($09)
  out(" ")
  str(s)
  out(EOL)
  return waitPrompt(maxTime)

PUB openWrite(s)                                    '' Open specified file for writing
  out($0E)
  out(" ")
  str(s)
  out(EOL)
  return waitPrompt(maxTime)

PUB closeWrite(s)                                   '' Close specified file being written
  out($0A)
  out(" ")
  str(s)
  out(EOL)
  return waitPrompt(maxTime)

PUB rename(sOld,sNew)                               '' Rename file or directory to new name
  out($0C)
  out(" ")
  str(sOld)
  out(" ")
  str(sNew)
  out(EOL)
  return waitPrompt(maxTime)

PUB delete(s)                                       '' Delete specified file
  out($07)
  out(" ")
  str(s)
  out(EOL)
  return waitPrompt(maxTime)

PUB fileSize(s) | f0, f1, f2, f3                    '' Return the file size
  out($01)
  out(" ")
  str(s)
  out(EOL)
  if recvChar(maxTime) <> EOL                       ' If no EOL, must be Command Failed
    abort Invalid
  getName(maxTime,@f0,16)                           ' Skip over the file name
  repeat 4
    result := result >> 8 | recvChar(maxTime) << 24 ' File size is 4 bytes, LSB first
  if not waitPrompt(maxTime)
    abort Invalid

PUB readFile(addr,size)                             '' Read specified # bytes from open file
  out($0B)
  out(" ")
  repeat 4
    size <-= 8
    out(size & $FF)
  out(EOL)
  repeat size
    byte[addr++] := recvChar(maxTime)
  return waitPrompt(maxTime)

PUB writeFile(addr,size)                            '' Write specified # bytes to open file
  out($08)
  out(" ")
  repeat 4
    size <-= 8
    out(size & $FF)
  out(EOL)
  repeat size
    out(byte[addr++])
  out(EOL)
  return waitPrompt(maxTime)

PUB directory                                       '' Echo directory information to display
  out($01)
  out(EOL)
  waitResponse(-maxTime,string(prompt,EOL))

PUB version                                         '' Echo version information to display
  out($13)
  out(EOL)
  waitResponse(-maxTime,string(prompt,EOL))

PUB identify                                        '' Echo disk information to display
  out($0F)
  out(EOL)
  waitResponse(-maxTime,string(prompt,EOL))

PUB online                                          '' Check if disk online
  out(EOL)
  return waitPrompt(maxTime)

PRI recvChar(ms) | t, time, limit                   '' Read data transfer until successful
  limit := (clkfreq / 1000) * ||ms
  time := cnt
  repeat while (t := transfer(true,false,0)) & 1
    maxTimeOut #>= cnt - time
    if (cnt - time) > limit                         ' Check for timeout
      abort TimeOut
  result := t >> 1 & $FF
  if ms < 0
    if result => " " or result == EOL               ' Echo to display
      dsp.out(result)
    else
      dsp.out("{")
      dsp.hex(result,2)
      dsp.out("}")

PRI sendChar(ms,c) | time, limit                    '' Write data transfer until successful
  limit := (clkfreq / 1000) * ||ms
  time := cnt
  repeat while transfer(false,false,c) & 1
    maxTimeOut #>= cnt - time
    if (cnt - time) > limit                         ' Check for timeout
      abort TimeOut
  if ms < 0
    if c => " " or c == EOL                         ' Echo to display
      dsp.out(c)
    else
      dsp.out("{")
      dsp.hex(c,2)
      dsp.out("}")

PUB getMaxTimeOut | mm, ss, ms                      '' Convert time to MMSSmmm
  ms := maxTimeOut / (clkfreq / 1000)
  ss := ms / 1000                                   ' mmm is milliseconds
  ms := ms // 1000
  mm := ss / 60                                     ' SS is seconds
  ss := ss // 60
  return mm * 100000 + ss * 1000 + ms               ' MM is minutes

PUB waitPrompt(ms) | t                              '' Wait for the standard prompt
  case t := recvChar(ms)
    prompt:                                         ' prompt,EOL indicates success
      result := true
    "N":                                            ' "ND",EOL indicates No Disk
      if (t := recvChar(ms)) <> "D"
        abort Invalid
      result := false
    "C":                                            ' "CF",EOL indicates Command Failed
      if (t := recvChar(ms)) <> "F"
        abort Invalid
      result := false
    other:                                          ' Any other response is invalid
      abort Invalid
  if (t := recvChar(ms)) <> EOL                     ' Must end with EOL
    abort Invalid

PUB waitResponse(ms,s)
  repeat until recvChar(ms) == byte[s]
  repeat strsize(s++) - 1
    if recvChar(ms) <> byte[s++]
      return false
  return true

PRI getHex(ms,size) | t, bits                       '' Receive a fixed width hex value
  bits := 0
  repeat size
    case t := recvChar(ms)
      "0".."9": result |= (t - "0") << bits
      "A".."F": result |= (t - "A" + 10) << bits
      "a".."f": result |= (t - "a" + 10) << bits
      other: abort ("?" << 8) | t
    bits += 4

PRI getName(ms,addr,size) | t
  repeat
    case t := recvChar(ms)
      " ":
        byte[addr] := 0
        return true
      EOL:
        byte[addr] := 0
        return false
      other:
        if size > 1
          byte[addr++] := t
          size--